@@ -12,7 +12,7 @@ package Socket::Class;
 our( $VERSION );
 
 BEGIN {
-	$VERSION = '1.23';
+	$VERSION = '2.0';
 	require XSLoader;
 	XSLoader::load( __PACKAGE__, $VERSION );
 	*say = \&writeline;
@@ -27,7 +27,7 @@ sub import {
 	@_ or return;
 	$Socket::Class::Const::VERSION
 		or require Socket::Class::Const;
-	&Exporter::export( 'Socket::Class::Const', $callpkg, @_ );
+	&Socket::Class::Const::export( $callpkg, @_ );
 }
 
 sub printf {
@@ -36,7 +36,7 @@ sub printf {
 		&Carp::croak( 'Usage: Socket::Class::printf(this,fmt,...)' );
 	}
 	my( $sock, $fmt ) = ( shift, shift );
-	return &write( $sock, sprintf( $fmt, @_ ) );
+	return $sock->write( sprintf( $fmt, @_ ) );
 }
 
 sub reconnect {
@@ -44,9 +44,10 @@ sub reconnect {
 		$Carp::VERSION or require Carp;
 		&Carp::croak( 'Usage: Socket::Class::reconnect(this,wait=0)' );
 	}
-	&close( $_[0] ) or return undef;
-	&wait( $_[0], $_[1] ) if $_[1];
-	&connect( $_[0] ) or return undef;
+	my $this = shift;
+	$this->close() or return undef;
+	$this->wait( $_[0] ) if $_[0];
+	$this->connect() or return undef;
 	return 1;
 }
 
@@ -67,7 +68,7 @@ Socket::Class - A class to communicate with sockets
 
 Socket::Class provides a simple, fast and efficient way to communicate with
 sockets.
-It operates outside of Perl's IO and socket layer. It can be used as a
+It operates outside of Perl IO and socket layer. It can be used as a
 B<replacement to IO::Socket>.
 Little parts of Bluetooth technology has been integrated. Please see below.
 
@@ -199,12 +200,12 @@ L<is_error|Socket::Class/is_error>
 
 =head2 Simple Internet Server
 
-  use Socket::Class qw($SOMAXCONN);
+  use Socket::Class qw(SOMAXCONN);
   
   # create a new socket on port 9999 and listen for clients 
   $server = Socket::Class->new(
        'local_port' => 9999,
-       'listen' => $SOMAXCONN,
+       'listen' => SOMAXCONN,
   ) or die Socket::Class->error;
   
   # wait for clients
@@ -300,8 +301,8 @@ L<is_error|Socket::Class/is_error>
 =item B<new ( [%arg] )>
 
 Creates a Socket::Class object, which is a reference to a newly created socket
-handle. I<new()> optionally takes arguments, these arguments are in key-value
-pairs.
+handle. I<new()> optionally takes arguments, these arguments must set as
+key-value pairs.
 
 =for formatter none
 
@@ -397,27 +398,27 @@ be allowed. Default is $SD_SEND.
 
   Num   Const         Description
   ----------------------------------------------------------------------
-  0     $SD_SEND      Disable sending on the socket.
-  1     $SD_RECEIVE   Disable receiving on the socket.
-  2     $SD_BOTH      Disable both sending and receiving on the socket.
+  0     SD_SEND       Disable sending on the socket.
+  1     SD_RECEIVE    Disable receiving on the socket.
+  2     SD_BOTH       Disable both sending and receiving on the socket.
 
 =for formatter perl
 
 B<Return Values>
 
-Returns a true value on succes or undef on failure.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Returns a true value on sucess or undef on failure.
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<Examples>
 
-  use Socket::Class qw($SD_BOTH);
+  use Socket::Class qw(SD_BOTH);
   
   $sock = Socket::Class->new( ... );
   
   ...
   
-  $sock->shutdown( $SD_BOTH );
+  $sock->shutdown( SD_BOTH );
   $sock->free();
 
 
@@ -464,9 +465,9 @@ The I<$port> parameter designates the port or channel on the local host.
 
 B<Return Values>
 
-Returns a true value on succes or undef on failure.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Returns a true value on sucess or undef on failure.
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<Examples>
 
@@ -490,15 +491,15 @@ so that retries may succeed.
 
 B<Return Values>
 
-Returns a true value on succes or undef on failure.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Returns a true value on sucess or undef on failure.
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<Examples>
 
-  use Socket::Class qw($SOMAXCONN);
+  use Socket::Class qw(SOMAXCONN);
   ...
-  $sock->listen( $SOMAXCONN )
+  $sock->listen( SOMAXCONN )
       or die $sock->error;
 
 
@@ -513,10 +514,11 @@ set_blocking(), 0 will be returned.
 
 B<Return Values>
 
-Returns a new socket class on succes or 0 on non-blocking mode and no new
-connection becomes available or UNDEF on failure. The error code can be
-retrieved with L<errno()|Socket::Class/errno> and the error string can
-be retrieved with L<error()|Socket::Class/error>. 
+Returns a new socket class on sucess or 0 on non-blocking mode and no new
+connection becomes available or UNDEF on failure.
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
+
 
 B<Examples>
 
@@ -587,9 +589,9 @@ Optionally timeout in milliseconds as floating point number.
 
 B<Return Values>
 
-Returns a TRUE value on succes or UNDEF on failure.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Returns a TRUE value on sucess or UNDEF on failure.
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<Examples>
 
@@ -604,9 +606,9 @@ reconnects the socket to the connection previously made.
 
 B<Return Values>
 
-Returns a true value on succes or undef on failure.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Returns a true value on sucess or undef on failure.
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<Examples>
 
@@ -646,11 +648,11 @@ The value of I<$flags> can be any combination of the following:
 
   Number  Constant         Description
   -------------------------------------------------------------
-  0x1     $MSG_OOB         Process OOB (out-of-band) data  
-  0x2     $MSG_PEEK        Peek at incoming message  
-  0x4     $MSG_DONTROUTE   Bypass routing, use direct interface  
-  0x8     $MSG_CTRUNC      Data completes record  
-  0x100   $MSG_WAITALL     Data completes transaction  
+  0x1     MSG_OOB          Process OOB (out-of-band) data  
+  0x2     MSG_PEEK         Peek at incoming message  
+  0x4     MSG_DONTROUTE    Bypass routing, use direct interface  
+  0x8     MSG_CTRUNC       Data completes record  
+  0x100   MSG_WAITALL      Data completes transaction  
 
 =for formatter perl
 
@@ -658,8 +660,8 @@ The value of I<$flags> can be any combination of the following:
 B<Return Values>
 
 Returns the number of bytes sent or undef if an error occured.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<Examples>
 
@@ -697,11 +699,11 @@ The value of I<$flags> can be any combination of the following:
 
   Number  Constant         Description
   ----------------------------------------------------------------------
-  0x1     $MSG_OOB         Process OOB (out-of-band) data  
-  0x2     $MSG_PEEK        Peek at incoming message  
-  0x4     $MSG_DONTROUTE   Bypass routing, use direct interface  
-  0x8     $MSG_CTRUNC      Data completes record
-  0x20    $MSG_TRUNC       Return the real length of the packet, even
+  0x1     MSG_OOB          Process OOB (out-of-band) data  
+  0x2     MSG_PEEK         Peek at incoming message  
+  0x4     MSG_DONTROUTE    Bypass routing, use direct interface  
+  0x8     MSG_CTRUNC       Data completes record
+  0x20    MSG_TRUNC        Return the real length of the packet, even
                            when it was longer than the passed buffer.
                            Only valid for packet sockets. 
 
@@ -710,8 +712,8 @@ The value of I<$flags> can be any combination of the following:
 B<Return Values>
 
 Returns the number of bytes received or undef on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<See Also>
 
@@ -740,17 +742,17 @@ The value of I<$flags> can be any combination of the following:
 
   Number  Constant         Description
   -------------------------------------------------------------
-  0x1     $MSG_OOB         Process OOB (out-of-band) data  
-  0x2     $MSG_PEEK        Peek at incoming message  
-  0x4     $MSG_DONTROUTE   Bypass routing, use direct interface  
+  0x1     MSG_OOB          Process OOB (out-of-band) data  
+  0x2     MSG_PEEK         Peek at incoming message  
+  0x4     MSG_DONTROUTE    Bypass routing, use direct interface  
 
 =for formatter perl
 
 B<Return Values>
 
 Returns the bytes sent to the remote host or undef on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<Examples>
 
@@ -797,12 +799,12 @@ I<$flags> parameter. Use the OR logic operator (|) to use more than one flag.
 
   Number  Constant         Description
   -----------------------------------------------------------------------
-  0x1     $MSG_OOB         Process OOB (out-of-band) data  
-  0x2     $MSG_PEEK        Receive data from the beginning of the receive
+  0x1     MSG_OOB          Process OOB (out-of-band) data  
+  0x2     MSG_PEEK         Receive data from the beginning of the receive
                            queue without removing it from the queue.
-  0x40    $MSG_DONTWAIT    With this flag set, the function returns even
+  0x40    MSG_DONTWAIT     With this flag set, the function returns even
                            if it would normally have blocked. 
-  0x100   $MSG_WAITALL     Block until at least len are received. However,
+  0x100   MSG_WAITALL      Block until at least len are received. However,
                            if a signal is caught or the remote host
                            disconnects, the function may return less data.
 
@@ -812,8 +814,8 @@ B<Return Values>
 
 Returns a packed address of the sender or 0 on non-blocking mode and no data
 becomes available or undef on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<Examples>
 
@@ -877,8 +879,8 @@ B<Return Values>
 
 Returns the number of bytes successfully written to the socket or UNDEF on
 error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<Notes>
 
@@ -918,8 +920,8 @@ The maximum number of bytes read is specified by the length parameter.
 B<Return Values>
 
 Returns number of bytes read, or undef on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<Examples>
 
@@ -943,8 +945,8 @@ B<Return Values>
 
 Returns the number of bytes successfully written to the socket, or undef on
 error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<Examples>
 
@@ -966,8 +968,8 @@ B<Return Values>
 
 Returns the number of bytes successfully written to the socket or UNDEF on
 error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<Examples>
 
@@ -978,19 +980,20 @@ B<Examples>
   $sock->write( sprintf( "%.3f", $number ) );
 
 
-=item B<say ( ... )>
-
 =item B<writeline ( ... )>
 
-Writes to the socket from the given string plus a newline char (\n).
-I<writeline> is a synonym for I<say>.
+=item B<say ( ... )>
+
+Writes to the socket from the given string plus a carriage return and a newline
+char (\r\n).
+I<say> is a synonym for I<writeline>.
 
 B<Return Values>
 
 Returns the number of bytes successfully written to the socket, or undef on
 error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<Examples>
 
@@ -999,13 +1002,13 @@ B<Examples>
 
 =item B<readline ()>
 
-Reads characters from the socket and stops at \n, \r or \0.
+Reads characters from the socket and stops at \r\n, \n\r, \n, \r or \0.
 
 B<Return Values>
 
 Returns a string value, or undef on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 
 =back
@@ -1027,8 +1030,8 @@ On 1 set blocking mode, on 0 set non-blocking mode.
 B<Return Values>
 
 Return a true value on succes, or undef on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 
 =item B<get_blocking ()>
@@ -1039,8 +1042,8 @@ B<Return Values>
 
 Return a TRUE value on blocking mode, or FALSE on non-blocking mode,
 or UNDEF on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 
 =item B<set_reuseaddr ( [$int] )>
@@ -1055,9 +1058,9 @@ On 1 enable reusaddr, on 0 disable reusaddr.
 
 B<Return Values>
 
-Return a TRUE value on succes or UNDEF on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Return a TRUE value on sucess or UNDEF on error.
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 
 =item B<get_reuseaddr ()>
@@ -1067,8 +1070,8 @@ Returns the current value of SO_REUSEADDR.
 B<Return Values>
 
 Return the value of SO_REUSEADDR or UNDEF on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 
 =item B<set_broadcast ( [$int] )>
@@ -1083,9 +1086,9 @@ On 1 enable reusaddr, on 0 disable reusaddr.
 
 B<Return Values>
 
-Return a TRUE value on succes or UNDEF on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Return a TRUE value on sucess or UNDEF on error.
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 
 =item B<get_broadcast ()>
@@ -1095,8 +1098,8 @@ Returns the current value of SO_BROADCAST.
 B<Return Values>
 
 Return the value of SO_BROADCAST or UNDEF on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 
 =item B<set_rcvbuf_size ( [$size] )>
@@ -1111,9 +1114,9 @@ The size of the receive buffer.
 
 B<Return Values>
 
-Return a TRUE value on succes or UNDEF on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Return a TRUE value on sucess or UNDEF on error.
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 
 =item B<get_rcvbuf_size ()>
@@ -1123,8 +1126,8 @@ Returns the current value of SO_RCVBUF.
 B<Return Values>
 
 Return the value of SO_RCVBUF or UNDEF on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 
 =item B<set_sndbuf_size ( [$size] )>
@@ -1139,9 +1142,9 @@ The size of the send buffer.
 
 B<Return Values>
 
-Return a TRUE value on succes or UNDEF on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Return a TRUE value on sucess or UNDEF on error.
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 
 =item B<get_sndbuf_size ()>
@@ -1151,8 +1154,8 @@ Returns the current value of SO_SNDBUF.
 B<Return Values>
 
 Return the value of SO_SNDBUF or UNDEF on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 
 =item B<set_timeout ( [$ms] )>
@@ -1167,9 +1170,9 @@ The timeout in milliseconds as floating point number.
 
 B<Return Values>
 
-Return a TRUE value on succes or UNDEF on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Return a TRUE value on sucess or UNDEF on error.
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 
 =item B<get_timeout ()>
@@ -1179,8 +1182,8 @@ Returns the current timeout.
 B<Return Values>
 
 Return the timeout in milliseconds or UNDEF on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 
 =item B<set_tcp_nodelay ( [$int] )>
@@ -1195,9 +1198,9 @@ On 1 disable the naggle algorithm, on 0 enable it.
 
 B<Return Values>
 
-Return a TRUE value on succes or UNDEF on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Return a TRUE value on sucess or UNDEF on error.
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 
 =item B<get_tcp_nodelay ()>
@@ -1207,8 +1210,8 @@ Returns the current value of TCP_NODELAY.
 B<Return Values>
 
 Return the value of TCP_NODELAY or UNDEF on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 
 =item B<set_option ( $level, $optname, $optval, ... )>
@@ -1237,25 +1240,25 @@ and are packed in the right format.
 
 B<Return Values>
 
-Return a TRUE value on succes or UNDEF on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Return a TRUE value on sucess or UNDEF on error.
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<Examples>
 
-  use Socket::Class qw($SOL_SOCKET $SO_LINGER $SO_RCVTIMEO);
+  use Socket::Class qw(SOL_SOCKET SO_LINGER SO_RCVTIMEO);
   
   $sock = Socket::Class->new( ... );
   
   # disable linger
-  $sock->set_option( $SOL_SOCKET, $SO_LINGER, 0, 0 );
+  $sock->set_option( SOL_SOCKET, SO_LINGER, 0, 0 );
   # same like
-  $sock->set_option( $SOL_SOCKET, $SO_LINGER, pack( 'S!S!', 0, 0 ) );
+  $sock->set_option( SOL_SOCKET, SO_LINGER, pack( 'S!S!', 0, 0 ) );
   
   # set rcv timeout to 0sec + 100000usec
-  $sock->set_option( $SOL_SOCKET, $SO_RCVTIMEO, 0, 100000 );
+  $sock->set_option( SOL_SOCKET, SO_RCVTIMEO, 0, 100000 );
   # or in milliseconds
-  $sock->set_option( $SOL_SOCKET, $SO_RCVTIMEO, 100 );
+  $sock->set_option( SOL_SOCKET, SO_RCVTIMEO, 100 );
 
 B<See Also>
 
@@ -1283,33 +1286,33 @@ A valid socket option.
 
   Option             Description
   -----------------------------------------------------------------------
-  $SO_DEBUG          Reports whether debugging information is being
+  SO_DEBUG           Reports whether debugging information is being
                      recorded.  
-  $SO_ACCEPTCONN     Reports whether socket listening is enabled.  
-  $SO_BROADCAST      Reports whether transmission of broadcast messages
+  SO_ACCEPTCONN      Reports whether socket listening is enabled.  
+  SO_BROADCAST       Reports whether transmission of broadcast messages
                      is supported.  
-  $SO_REUSEADDR      Reports whether local addresses can be reused.  
-  $SO_KEEPALIVE      Reports whether connections are kept active with
+  SO_REUSEADDR       Reports whether local addresses can be reused.  
+  SO_KEEPALIVE       Reports whether connections are kept active with
                      periodic transmission of messages. If the connected
                      socket fails to respond to these messages, the
                      connection is broken and processes writing to that
                      socket are notified with a SIGPIPE signal.  
-  $SO_LINGER         Reports whether the socket lingers on close()
+  SO_LINGER          Reports whether the socket lingers on close()
                      if data is present.  
-  $SO_OOBINLINE      Reports whether the socket leaves out-of-band data
+  SO_OOBINLINE       Reports whether the socket leaves out-of-band data
                      inline.  
-  $SO_SNDBUF         Reports send buffer size information.  
-  $SO_RCVBUF         Reports recieve buffer size information.  
-  $SO_ERROR          Reports information about error status and clears it.  
-  $SO_TYPE           Reports the socket type.  
-  $SO_DONTROUTE      Reports whether outgoing messages bypass the standard
+  SO_SNDBUF          Reports send buffer size information.  
+  SO_RCVBUF          Reports recieve buffer size information.  
+  SO_ERROR           Reports information about error status and clears it.  
+  SO_TYPE            Reports the socket type.  
+  SO_DONTROUTE       Reports whether outgoing messages bypass the standard
                      routing facilities.  
-  $SO_RCVLOWAT       Reports the minimum number of bytes to process for
+  SO_RCVLOWAT        Reports the minimum number of bytes to process for
                      socket input operations. ( Defaults to 1 )  
-  $SO_RCVTIMEO       Reports the timeout value for input operations.  
-  $SO_SNDLOWAT       Reports the minimum number of bytes to process for
+  SO_RCVTIMEO        Reports the timeout value for input operations.  
+  SO_SNDLOWAT        Reports the minimum number of bytes to process for
                      socket output operations.  
-  $SO_SNDTIMEO       Reports the timeout value specifying the amount of
+  SO_SNDTIMEO        Reports the timeout value specifying the amount of
                      time that an output function blocks because flow
                      control prevents data from being sent.  
 
@@ -1321,24 +1324,24 @@ Returns the value of the given option, or UNDEF on error.
 If the size of the value equals the size of int the value will be unpacked as
 integer.
 For SO_LINGER, SO_RCVTIMEO and SO_SNDTIMEO the value will be unpacked also.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<Examples>
 
-  use Socket::Class qw($SOL_SOCKET $SO_LINGER $SO_RCVTIMEO);
+  use Socket::Class qw(SOL_SOCKET SO_LINGER SO_RCVTIMEO);
   
   $sock = Socket::Class->new( ... );
   
   # get linger
   ($l_onoff, $l_linger) =
-      $sock->get_option( $SOL_SOCKET, $SO_LINGER );
+      $sock->get_option( SOL_SOCKET, SO_LINGER );
   
   # get rcv timeout
   ($tv_sec, $tv_usec) =
-      $sock->get_option( $SOL_SOCKET, $SO_RCVTIMEO );
+      $sock->get_option( SOL_SOCKET, SO_RCVTIMEO );
   # or in milliseconds
-  $ms = $sock->get_option( $SOL_SOCKET, $SO_RCVTIMEO );
+  $ms = $sock->get_option( SOL_SOCKET, SO_RCVTIMEO );
 
 B<See Also>
 
@@ -1446,8 +1449,8 @@ If no address is specified the remote address of the socket is used.
 B<Return Values>
 
 Returns the first hostname found, or UNDEF on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<Examples>
 
@@ -1478,8 +1481,8 @@ The host name as a string.
 B<Return Values>
 
 Returns the address of the host, or UNDEF on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<Examples>
 
@@ -1505,31 +1508,31 @@ dotted-decimal IPv4 address or an IPv6 hex address.
 
 I<$service>
 
-A service name or port number, or undef if it is unused.
+A service name or port number, or undef to get all services.
 
 I<$family>
 
-The address family as name or number, or undef if it is unused.
+The address family as name or number, or undef to get all families.
 
 I<$proto>
 
-The protocol type as name or number, or undef if it is unused.
+The protocol type as name or number, or undef to get all protocols.
 
 I<$type>
 
-The socket type as name or number, or undef if it is unused.
+The socket type as name or number, or undef to get all socket types.
 
 I<$flags>
 
 Flags that indicate options used.
-See also L<getaddrinfo() flags|Socket::Class::Const/getaddrinfo()_flags>
-in L<Socket::Class::Const|Socket::Class::Const>
+See L<getaddrinfo() flags|Socket::Class::Const/"Flags for getaddrinfo()">
+in the L<Socket::Class::Const|Socket::Class::Const> module
 
 B<Return Values>
 
 Returns an array of hashes with address information, or undef on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 Address information structure:
 
@@ -1596,20 +1599,21 @@ A service name or port number.
 
 I<$paddr>
 
-A packed host address. See also C<pack_addr()>, C<getaddrinfo()>
+A packed host address. See also L<pack_addr()|Socket::Class/pack_addr>,
+L<getaddrinfo()|Socket::Class/getaddrinfo>
 
 I<$flags>
 
 A value used to customize processing of the I<getnameinfo> function.
-See also L<getnameinfo() flags|Socket::Class::Const/getnameinfo()_flags>
-in L<Socket::Class::Const|Socket::Class::Const>
+See L<getnameinfo() flags|Socket::Class::Const/Flags for getnameinfo()>
+in the L<Socket::Class::Const|Socket::Class::Const> module
 
 B<Return Values>
 
 Returns the hostname in scalar context, or hostname and service in array
 context, or undef on error. 
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<Examples>
 
@@ -1650,13 +1654,13 @@ Gets the amount of data that is available to be read.
 B<Return Values>
 
 Returns the number of bytes that is available to be read, or undef on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<Remarks>
 
-On blocking sockets the function can block indefinitely.
-You should call L<is_readable()|Socket::Class/is_readable> before running this
+On blocking sockets the function can block infinitely. In this case you
+should call L<is_readable()|Socket::Class/is_readable> before running the
 function.
 
 B<Examples>
@@ -1687,8 +1691,8 @@ If the value is undef (no timeout), I<is_readable()> can block indefinitely.
 B<Return Values>
 
 Return 1 if the socket is readable, or 0 if it is not, or UNDEF on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 
 =item B<is_writable ( [$timeout] )>
@@ -1707,8 +1711,8 @@ If the value is undef (no timeout), I<is_writable()> can block indefinitely.
 B<Return Values>
 
 Return 1 if the socket is writable, or 0 if it is not, or UNDEF on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 
 =item B<select ( [$read [, $write [, $except [, $timeout]]]] )>
@@ -1746,8 +1750,8 @@ B<Return Values>
 
 Returns a number between 0 to 3 which indicates the parameters set to TRUE, or
 UNDEF on error.
-The error code can be retrieved with L<errno()|Socket::Class/errno>
-and the error string can be retrieved with L<error()|Socket::Class/error>. 
+Use L<errno()|Socket::Class/errno> and L<error()|Socket::Class/error>
+to retrieve the error code and message. 
 
 B<Remarks>
 
@@ -1833,14 +1837,14 @@ B<Return Values>
 
 =for formatter none
 
-  Number   Constant         Description
+  Number   Constant             Description
   ---------------------------------------------------
-  0        $SOS_INIT        Socket is created
-  1        $SOS_BOUND       Socket is bound
-  2        $SOS_LISTEN      Socket is listening
-  3        $SOS_CONNECTED   Socket is connected
-  4        $SOS_CLOSED      Socket is closed
-  99       $SOS_ERROR       Socket got an error on last send or receive
+  0        SC_STATE_INIT        Socket is created
+  1        SC_STATE_BOUND       Socket is bound
+  2        SC_STATE_LISTEN      Socket is listening
+  3        SC_STATE_CONNECTED   Socket is connected
+  4        SC_STATE_CLOSED      Socket is closed
+  99       SC_STATE_ERROR       Socket got an error on last send or receive
 
 =for formatter perl
 
@@ -1867,7 +1871,7 @@ B<Parameters>
 
 I<$ms>
 
-The number of milliseconds to sleep.
+The number of milliseconds to sleep as floating point number.
 
 
 =back
@@ -1913,10 +1917,11 @@ from the last occurred error.
       'reuseaddr' => 1,
   ) or die Socket::Class->error;
   
-  # catch interrupt signals to provide clean shutdown
+  # catch interrupt signals for a clean shutdown
   $SIG{'INT'} = \&quit;
   #$SIG{'TERM'} = \&quit;
   
+  # create the server thread
   threads->create( \&server_thread, $Server );
   
   while( $RUNNING ) {
@@ -1932,7 +1937,9 @@ from the last occurred error.
       my( $thread );
       $RUNNING = 0;
       foreach $thread( threads->list ) {
-          $thread->join();
+          eval {
+              $thread->join();
+          };
       }
       $Server->free();
       exit( 0 );
@@ -1989,11 +1996,59 @@ from the last occurred error.
       return 1;
   }
 
+=head1 XS / C API
+
+The module provides a C interface for extension writers.
+
+B<Example XS>
+
+=for formatter cpp
+
+  #include <Socket/Class/mod_sc.h>
+  
+  /* global pointer to the socket class interface */
+  mod_sc_t *g_mod_sc;
+  
+  MODULE = MyModule		PACKAGE = MyModule
+  
+  BOOT:
+  {
+      SV **psv;
+      psv = hv_fetch(PL_modglobal, "Socket::Class", 13, 0);
+      if (psv == NULL)
+          croak("Socket::Class 2.0 or higher required");
+      g_mod_sc = INT2PTR(mod_sc_t *, SvIV(*psv));
+  }
+  
+  void
+  test()
+  PREINIT:
+      sc_t *socket;
+      char *args[4];
+      int r;
+      SV *sv;
+  PPCODE:
+      args[0] = "local_port";
+      args[1] = "8080";
+      args[2] = "listen";
+      args[3] = "10";
+      r = g_mod_sc->sc_create(args, 4, &socket);
+      if (r != SC_OK)
+          croak(g_mod_sc->sc_get_error(NULL));
+      g_mod_sc->sc_create_class(socket, NULL, &sv);
+      ST(0) = sv_2mortal(sv);
+      XSRETURN(1);
+
+=for formatter perl
+
+See I<[sitearch]/auto/Socket/Class/mod_sc.h> for the definition
+and look to the source code of I<Class.xs> for an implementation.
+
 =head1 AUTHORS
 
-Written and maintained by Christian Mueller
+Navalla org., Christian Mueller, L<http://www.navalla.org/>
 
-=head1 COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
 The Socket::Class module is free software. You may distribute under the
 terms of either the GNU General Public License or the Artistic
