@@ -393,8 +393,10 @@ extern sc_global_t global;
 /* wont work !? */
 #define my_set_errno(code) \
 	do { \
+		_debug( "set errno %d\n", code ); \
 		errno = code; \
 		_set_errno( code ); \
+		_set_doserrno( code ); \
 		SetLastError( (DWORD) code ); \
 	} while( 0 )
 #else
